/*
 *----------------------------------------------------------------------------
 * Copyright (c) 2001, 2002, C.S. Peron
 * All rights reserved.
 *----------------------------------------------------------------------------
 *
 *  * Redistribution and use in source and binary forms, with or without 
 *    modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this 
 *    list of conditions and the following disclaimer.
 *
 *  * Redistributions in binary form must reproduce the above 
 *    copyright notice, this list of conditions and the following disclaimer 
 *    in the documentation and/or other materials provided with the distribution.
 *
 *  * Neither the name of the ipex developers nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software 
 *    without specific prior written permission. 
 *
 *----------------------------------------------------------------------------
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *----------------------------------------------------------------------------
 */

#ifndef lint
static const char rcsid[] =
        "@(#) $Header: /usr/cvs/ipex24000/igmp_print.c,v 1.4 2004/08/05 22:57:56 modulus Exp $";
#endif

#include "ipex_includes.h"

int
print_igmp(register u_char *b, register struct pcap_pkthdr *h,
    register u_char *p)
{
	char *d;
	int len;
	register opt_t *x = (opt_t *)uap;
	register struct ip *ip = (struct ip *)(p + x->loffset);
	unsigned ip_hl = (ip->ip_hl * 0x4);
	unsigned ip_off = ntohs(ip->ip_off);
        u_32_t *sa, *da;
	unsigned frag =  ip_off & (IP_MF|IP_OFFMASK);
        unsigned frag_off = frag ? (ip_off & IP_OFFMASK) * 8:0;
	register struct igmp *igmph = (struct igmp *)(((char *)ip) + ip_hl);
	unsigned igmp_off = frag ? 0 : sizeof(struct igmp);
        struct ehdr *mhdr = NULL; 

	d = ((char *)igmph) + (igmp_off);
	len = (ntohs(ip->ip_len) - ip_hl - igmp_off);
        sa = (u_32_t *)&ip->ip_src;
        da = (u_32_t *)&ip->ip_dst;
        if (uap->flags & FLINK)
           mhdr = process_link_headers(b, h, p);
	printf("%s %s%s ",
	    time_stamp(&h->ts, x->thiszone),
	    ((uap->flags & FLINK) ? mhdr->smac : "" ),
	    hostname(x->flags, 4, sa));
	printf("> %s%s igmp ",
	    ((uap->flags & FLINK) ? mhdr->smac : "" ),
	    hostname(x->flags, 4, da));
	if (frag) {
		printf(" %d:%d@%d%s\n",
		    ntohs(ip->ip_id),
		    len, frag_off,
		    frag_off ? "" : "+");
	} else {
		printf( " (DF)\n");
        }
	uap->pcount += 1;
	return (0);
}

int
generic_print(register u_char *b, register struct pcap_pkthdr *h,
    register u_char *p)
{
	register opt_t *x = (opt_t *)b;
	register struct ip *ip = (struct ip *)(p + x->loffset);
	u_32_t *sa, *da;
	struct ehdr *mhdr = NULL;

	sa = (u_32_t *)&ip->ip_src;
	da = (u_32_t *)&ip->ip_dst;
        if (uap->flags & FLINK)
		mhdr = process_link_headers(b, h, p);
	printf("%s %s%s ",
	    time_stamp(&h->ts, x->thiszone),
	    ((uap->flags & FLINK) ? mhdr->smac : "" ),
	    hostname(x->flags, 4, sa));
	printf("> %s%s %s <id=%u tos=0x%02x ttl=%u>\n",
	    ((uap->flags & FLINK) ? mhdr->smac : "" ),
	    hostname(x->flags, 4, da),
	    getproto(ip->ip_p), ntohs(ip->ip_id), ip->ip_tos, ip->ip_ttl);
	ascii_print_with_offset(p, h->len, 0x0, x->fd);
	uap->pcount += 1;
        return (0);
}
