/*
 *----------------------------------------------------------------------------
 * Copyright (c) 2001, 2002, C.S. Peron
 * All rights reserved.
 *----------------------------------------------------------------------------
 *
 *  * Redistribution and use in source and binary forms, with or without 
 *    modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this 
 *    list of conditions and the following disclaimer.
 *
 *  * Redistributions in binary form must reproduce the above 
 *    copyright notice, this list of conditions and the following disclaimer 
 *    in the documentation and/or other materials provided with the distribution.
 *
 *  * Neither the name of the ipex developers nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software 
 *    without specific prior written permission. 
 *
 *----------------------------------------------------------------------------
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *----------------------------------------------------------------------------
 */

#if (!defined(IPEX_H_))
#define IPEX_H_

#if defined(linux) && !_BSD_SOURCE
#define _BSD_SOURCE
#endif

#include <netinet/in.h>
#include <pcap.h>
#include <sys/types.h>
#include <sys/time.h>
#include <time.h>

/*
 * setup bit masks for our options.
 */					/* our bit masks	*/
#define	FDHEX	0x1			/* dump hex? 	*/
#define FDVER	((FDHEX) << (1))	/* dump version?	*/
#define FINWL	((FDVER) << (1))	/* insert newline */
#define FASYS	((FINWL) << (1))	/* alert syslogd? */
#define FDAMO	((FASYS) << (1))	/* daemon mode	*/
#define FJHDR	((FDAMO) << (1))	/* just headers? */
#if (defined(USE_DNS))
#define	FPDNS	((FJHDR) << (1))	/* print dns?	*/
#define FPHEX	((FPDNS) << (1))	/* print addrs in hex?*/
#define FPNTA	((FPHEX) << (1))	/* print addr in x.x.x.x */
#endif	/* USE_DNS */
#define FPCOP	((FPNTA) << (1))	/* pcap compile optizer ? */
#define FRCFD	((FPCOP) << (1))	/* file which contains regex RC's */
#define FSYSL	((FRCFD) << (1))	/* Inform system log when -R used */
#define FLINK	((FSYSL) << (1))	/* Print link addresses */
#define FGNUR	((FLINK) << (1))	/* use gnu regexp? */
#define FBPFD	((FGNUR) << (1))	/* dump struct bpf_program? */
#define FDUMP   ((FBPFD) << (1))	/* dump entire packet */
#define PTRACK	((FDUMP) << (1))	/* Do packet/TCP packet accounting */
#define	LPORTS	((PTRACK) << (1))	/* consult /etc/services for ports */
#define TIMER	((LPORTS) << (1))

/*
 * define our macros to check flags.
 * its basic; if the flag is set, evalulate
 * to true (1), else false (0)
 */

#define	IS_FDHEX(a)	((a & FDHEX) ? (1) : (0))
#define IS_FDVER(a)	((a & FDVER) ? (1) : (0))
#define IS_FINWL(a)	((a & FINWL) ? (1) : (0))
#define IS_FASYS(a)	((a & FASYS) ? (1) : (0))
#define IS_FDAMO(a)	((a & FDAMO) ? (1) : (0))
#define IS_FJHDR(a)	((a & FJHDR) ? (1) : (0))
#define IS_FRCFD(a)	((a & FRCFD) ? (1) : (0))
#define IS_FSYSL(a)	((a & FSYSL) ? (1) : (0))
#define IS_FLINK(a)	((a & FLINK) ? (1) : (0))
#define IS_FGNUR(a)	((a & FGNUR) ? (1) : (0))
#define IS_FBPFD(a)	((a & FBPFD) ? (1) : (0))
#define IS_FDUMP(a)	((a & FDUMP) ? (1) : (0))
#if (defined(USE_DNS))
#define IS_FPDNS(a)	((a & FPDNS) ? (1) : (0))
#define IS_FPHEX(a)	((a & FPHEX) ? (1) : (0))
#define IS_FPNTA(a)	((a & FPNTA) ? (1) : (0))
#define	IS_LPORTS(a)	((a & LPORTS) ? (1) : (0))

#endif 

struct signodes {
   unsigned char *signature;
   char *debug;
   int sigid;
   int siglen;
   int loglevel;
   int (*handler)();
   struct signodes *next, *prev;
};

struct dump_info {
	char		*WFileName;
        pcap_t		*pd;
        pcap_dumper_t	*p;
};

struct ehdr {
   char smac[20];
   char dmac[20];
};

/*
 * define our option structure
 */

typedef struct _opt_t {
  int	Cflag;
  struct timeval tm;	/* timestamps */
  int32_t	thiszone;	/* time zone */
  unsigned int flags;	/* flags that are set on the command line */
  int	uflag;		/* set uid on daemon? */
  int	pcount;		/* packet's written counter */
  int	fd;		/* file descriptor (avoiding globals) */
  int	loffset;	/* data link offset */
  int	cflag;		/* how many packets to capture? */
  int 	Oflag;		/* run pcap code optimzer ? */
  char	*fflag;		/* we want to use a config file? */
  char	*oflag;		/* we want to dump to stdout or no? */
  char	*pflag;		/* supply pcap filter logic here? */
  char	*rflag;		/* regex RC's? */
  char	*Rflag;		/* regex RC's from a file? */
  char	*iflag;		/* network interface? */
  char  *Eflag;
  char	*Pflag;		/* sniff via process? */
  char	*Fflag;		/* read packets from this file instead of interface */
  char	*wflag;		/* write raw packets to this file */
  char  *Bflag;		/* binary signature file for light weight IDS-like functionality */
  int	Tflag;		/* TCP session timeout */
} opt_t;

struct datespec *dsp;

/*
 * this is going to be a copy of the uap struct
 * defined globally.. for clean up.
 */
opt_t *uap;

/*
 * define a malloc error macro.
 * strictly for appearence and
 * compactness.
 */

#define	MALLOC_FAIL() 				\
	(void) fprintf(stderr, 			\
		"fatal: malloc(3): failed.\n");

typedef unsigned int    u_32_t;
#if (!defined(MAXHOSTNAMELEN))
#define MAXHOSTNAMELEN 256
#endif	/* MAXHOSTNAMELEN */

/*
 * default snapshot length. keep the amount of
 * unwanted data down. This still allows for 
 * all the neccessary headers.
 */
#define DEFAULT_SNAPLEN 68
#define ETHHDR_SIZE     14
#define PPPHDR_SIZE     4
#define SLIPHDR_SIZE    16
#define RAWHDR_SIZE     0
#define LOOPHDR_SIZE    4
#define FDDIHDR_SIZE    21
#ifndef IP_OFFMASK
#define IP_OFFMASK 0x1fff
#if (defined(HAVE_NET_IF_PFLOG_H)) 
#define PFLOG_HDRLEN        sizeof(struct pfloghdr)
#endif
#endif

/*
 * OpenBSD doesn't have LOG_SECURITY
 */
#ifndef LOG_SECURITY
#define LOG_SECURITY LOG_AUTHPRIV
#endif

#endif	/* IPEX_H_ */
