/*
 *----------------------------------------------------------------------------
 * Copyright (c) 2001, 2002, C.S. Peron
 * All rights reserved.
 *----------------------------------------------------------------------------
 *
 *  * Redistribution and use in source and binary forms, with or without
 *    modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 *
 *  * Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following disclaimer
 *    in the documentation and/or other materials provided with the distribution.
 *
 *  * Neither the name of the ipex developers nor the names of its contributor
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 *----------------------------------------------------------------------------
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *----------------------------------------------------------------------------
 */

#ifndef lint
static const char rcsid[] =
        "@(#) $Header: /usr/cvs/ipex24000/tcp_session.c,v 1.4 2004/07/29 15:07:30 modulus Exp $";
#endif

#include "ipex_includes.h"

#define HPARENT(npos) ((int)(((npos) - 1) / 2))
#define HLEFT(npos) (((npos) * 2) + 1)
#define HRIGHT(npos) (((npos) * 2) + 2)

TAILQ_HEAD(tailhead, conq) head = TAILQ_HEAD_INITIALIZER(head);
typedef struct {
	unsigned int values[5];
} uint128_t;

#ifndef TAILQ_FOREACH_SAFE
#define TAILQ_FOREACH_SAFE(var, head, field, tvar)			\
	for ((var) = TAILQ_FIRST((head));				\
	    (var) && ((tvar) = TAILQ_NEXT((var), field), 1);		\
	    (var) = (tvar))
#endif

char *
chomp(char *s)
{
	char *p;

	while((p = strchr(s, '.'))) {
		*p = '\0';
		p++;
	}
	return (s);
}

char *
tcp_genfname(u_char *data, struct pcap_pkthdr *h, u_char *p)
{
	struct ip *ip = (struct ip *)(p + uap->loffset);
	unsigned ip_hl = (ip->ip_hl * 0x4);
	struct tcphdr *tcp = ((struct tcphdr *)(((char *)ip) + ip_hl));
	static char fname[256];
	uint128_t hash;

	memcpy(&hash.values[0], &ip->ip_src,
	    sizeof(struct in_addr));
	memcpy(&hash.values[1], &ip->ip_dst,
	    sizeof(struct in_addr));
	hash.values[2] = tcp->th_sport;
	hash.values[3] = tcp->th_dport;
	hash.values[4] = time(NULL);
	snprintf(fname, sizeof(fname), "%08x%08x%04x%04x%08x.dmp",
	    hash.values[0], hash.values[1],
	    hash.values[2], hash.values[3],
	    hash.values[4]);
	return (fname);
}

void
disassemble_fname(char *filename)
{
	uint128_t hash;
	char true[256];
	int fd;
	off_t nbytes;
	struct stat sb;

	snprintf(true, sizeof(true), "%s.dmp", filename);
	if ((fd = open(true, O_RDONLY)) < 0) 
		err(1, "open failed");
	if (fstat(fd, &sb) < 0)
		err(1, "stat failed");
	nbytes = sb.st_size;
	close(fd);
	sscanf(filename, "%08x%08x%04x%04x%08x",
	    &hash.values[0], &hash.values[1],
	    &hash.values[2], &hash.values[3],
	    &hash.values[4]);
	printf(";;\n");
	printf(";; Filename: %s.dmp\n", filename);
	printf(";; Host(s)/Port(s): %s:%u and",
	    hostname(uap->flags, 4, &hash.values[0]),
	    htons(hash.values[2]));
	printf(" %s:%u\n",
	    hostname(uap->flags, 4, &hash.values[1]),
	    htons(hash.values[3]));
	printf(";; Time: %s", ctime((time_t *)&hash.values[4]));
	printf(";; Size: %llu bytes\n\n", nbytes);
}

int
compare(struct conq *conep, struct conq *ctwop)
{
	if (conep->hits > ctwop->hits)
		return (1);
	else
		return(-1);
}

void
heap_init(Heap *heap, int (*cmp)(const void *key1, const void *key2),
    void (*unalloc)(void *data))
{
	TAILQ_INIT(&head);
}

int
heap_extract(Heap *heap, void **data)
{
	int ipos, lpos, rpos, mpos;
	void *save, *tmp;

	if (HEAP_SIZE(heap) == 0)
		return(-1);
	*data = heap->btree[0];
	save = heap->btree[HEAP_SIZE(heap) - 1];
	if (HEAP_SIZE(heap) - 1 > 0) {
		tmp = memrealloc(heap->btree, (HEAP_SIZE(heap) - 1) *
		    sizeof(void *));
		heap->btree = tmp;
		heap->size--;
	} else {
		free(heap->btree);
		heap->btree = NULL;
		heap->size = 0;
		return (0);
	}
	heap->btree[0] = save;
	ipos = 0;
	lpos = HLEFT(ipos);
	rpos = HRIGHT(ipos);
	for (;;) {
		lpos = HLEFT(ipos);
		rpos = HRIGHT(ipos);
		if (lpos < HEAP_SIZE(heap) &&
		    heap->cmp(heap->btree[lpos], heap->btree[ipos]) > 0)
			mpos = lpos;
		else
			mpos = ipos;
		if (rpos < HEAP_SIZE(heap) &&
		    heap->cmp(heap->btree[rpos], heap->btree[mpos]) > 0)
			mpos = rpos;
		if (mpos == ipos)
			break;
		else {
			tmp = heap->btree[mpos];
			heap->btree[mpos] = heap->btree[ipos];
			heap->btree[ipos] = tmp;
			ipos = mpos;
		}
	}
	return (0);
}

int
tcp_sessionreaper(void)
{
	struct conq *cq, *cq2;
	time_t now;

	now = time(0);
	TAILQ_FOREACH_SAFE(cq, &head, glue, cq2) {
		if ((now - cq->lastwrite) > uap->Tflag) {
			printf("NOTE: closing TCP session file %s (timeout)\n",
			    cq->fname);
			TAILQ_REMOVE(&head, cq, glue);
			pcap_dump_close(cq->dumper);
			free(cq->rev);
			free(cq);
		}
	}
	return (0);
}

int
close_tcpsession(struct conq *cqp)
{
	/*
	 * Close the dump file and schedule it for removal
	 * from the TCP session queue.
	 */
	printf("NOTE: closing TCP session file %s (RST/FIN)\n",
	    cqp->fname);
	pcap_dump_close(cqp->dumper);
	cqp->dead = 1;
	return (0);
}

int
get_tcpsession(u_char *packet, Heap *heap, struct conq **cq)
{
	struct ip *ip = (struct ip *)(packet + uap->loffset);
	unsigned ip_hl = (ip->ip_hl * 0x4);
	register struct tcphdr *tcp = ((struct tcphdr *)(((char *)ip) + ip_hl));
	int i;
	struct conq *ctx;

	TAILQ_FOREACH(ctx, &head, glue) {
		if ((memcmp(&ip->ip_src, &ctx->ip_src,
		    sizeof(struct in_addr)) == 0) &&
		    (tcp->th_sport == ctx->sport)) {
			if (ctx->dead) {
				return (-1);
			}
			*cq = ctx;
			return (1);
		}
		if ((memcmp(&ip->ip_src, &ctx->rev->ip_src,
		    sizeof(struct in_addr)) == 0) &&
		    (tcp->th_sport == ctx->rev->sport)) {
			if (ctx->dead) {
				return (-1);
			}
			*cq = ctx;
			return (1);
		}
	}
	return (0);
}

void
heap_destory(Heap *heap)
{
	struct conq *cq, *cq2;

	TAILQ_FOREACH_SAFE(cq, &head, glue, cq2) {
		TAILQ_REMOVE(&head, cq, glue);
		free(cq->rev);
		free(cq);
	}
}

int
heap_insert(Heap *heap, const void *data)
{
	struct conq *cq;

	cq = (struct conq *)data;
	TAILQ_INSERT_HEAD(&head, cq, glue);
	return (0);
}

void
handle_sessions(struct pcap_pkthdr *h, u_char *p)
{
	struct ip *ip = (struct ip *)(p + uap->loffset);
	int error;
	char *fname;
	struct conq *session;
	extern pcap_t *pd;

	tcp_sessionreaper();
	if (ip->ip_p == IPPROTO_TCP) {
		unsigned ip_hl = (ip->ip_hl * 0x4);
		struct tcphdr *tcp = ((struct tcphdr *)(((char *)ip)
			+ ip_hl));

		error = get_tcpsession(p, 0, &session);
		if (error == 0) {
			session = memalloc(sizeof(struct conq));
			session->hits++;
			fname = tcp_genfname(0, h, p);
			if ((session->dumper = pcap_dump_open(pd,
			    fname)) == NULL) {
				fprintf(stderr,
				    "fatal: pcap_dump_open: %s",
				    pcap_geterr(pd));
				cleanup();
			}
			session->dead = 0;
			session->fname = fname;
			memcpy(&session->ip_src, &ip->ip_src,
			    sizeof(struct in_addr));
			memcpy(&session->ip_dst, &ip->ip_dst,
			    sizeof(struct in_addr));
			memcpy(&session->sport,  &tcp->th_sport,
			    sizeof(u_short));
			memcpy(&session->dport,  &tcp->th_dport,
			    sizeof(u_short));
			session->rev = memalloc(sizeof(struct conq));
			memcpy(&session->rev->ip_dst, &ip->ip_src,
			    sizeof(struct in_addr));
			memcpy(&session->rev->ip_src, &ip->ip_dst,
			    sizeof(struct in_addr));
			memcpy(&session->rev->sport,  &tcp->th_dport,
			    sizeof(u_short));
			memcpy(&session->rev->dport,  &tcp->th_sport,
			    sizeof(u_short));
			heap_insert(0, session);
			pcap_dump((u_char *)session->dumper, h, p);
			session->lastwrite = time(0);
			fflush((FILE *)session->dumper);
			return;
		} else if (error == 1) {
			if (tcp->th_flags & (TH_FIN | TH_RST)) {
				/*
				 * write packet which closed the session
				 */
				pcap_dump((u_char *)session->dumper, h, p);
				close_tcpsession(session);
				return;
			}
			session->hits++;
			pcap_dump((u_char *)session->dumper, h, p);
			session->lastwrite = time(0);
			fflush((FILE *)session->dumper);
			return;
		} else if (error < 0)
			return;
	}
	return;
}
