/*
 *----------------------------------------------------------------------------
 * Copyright (c) 2001, 2002, C.S. Peron
 * All rights reserved.
 *----------------------------------------------------------------------------
 *
 *  * Redistribution and use in source and binary forms, with or without 
 *    modification, are permitted provided that the following conditions
 *    are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  * Redistributions in binary form must reproduce the above 
 *    copyright notice, this list of conditions and the following disclaimer 
 *    in the documentation and/or other materials provided with the
 *    distribution.
 *
 *  * Neither the name of the ipex developers nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software 
 *    without specific prior written permission. 
 *
 *----------------------------------------------------------------------------
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *----------------------------------------------------------------------------
 */

#ifndef lint
static const char rcsid[] = \
    "@(#) $Header: /usr/cvs/ipex24000/udp_print.c,v 1.3 2004/08/05 23:01:25 modulus Exp $";
#endif

#include "ipex_includes.h"

int
print_udp(u_char *b, struct pcap_pkthdr *h, u_char *p)
{
	char *d;
	int len;
	opt_t *x = (opt_t *)uap;
	struct ip *ip = (struct ip *)(p + x->loffset);
	unsigned ip_hl = (ip->ip_hl * 0x4);
	unsigned ip_off = ntohs(ip->ip_off);
	unsigned frag =  ip_off & (IP_MF|IP_OFFMASK);
	unsigned frag_off = frag ? (ip_off & IP_OFFMASK) * 8:0;
	struct udphdr *udp = (struct udphdr *)(((char *)ip) + ip_hl);
	unsigned udp_off = (frag) ? 0 : sizeof(struct udphdr);
        struct ehdr *mhdr = NULL;
        u_32_t *sa, *da;

	d = ((char *)udp) + (udp_off);
	len = (ntohs(ip->ip_len) - ip_hl - udp_off);
        sa = (u_32_t *)&ip->ip_src;
        da = (u_32_t *)&ip->ip_dst;
        if (uap->flags & FLINK)
		mhdr = process_link_headers(b, h, p);
#if (defined(USE_DUMB_UDP_HEADERS))
	printf("%s %s%s:%s ",
	    time_stamp(&h->ts, x->thiszone),
	    ((uap->flags & FLINK) ? mhdr->smac : ""),
	    hostname(x->flags, 4, sa),
	    portname(IS_LPORTS(uap->flags), "udp", ntohs(udp->source)));
	printf( "> %s%s:%s udp",
	    ((uap->flags & FLINK) ? mhdr->dmac : ""),
	    hostname(x->flags, 4, da),
	    portname(IS_LPORTS(uap->flags), "udp", ntohs(udp->dest)));
#else
	printf("%s %s%s:%s ",
	    time_stamp(&h->ts, x->thiszone),
	    ((uap->flags & FLINK) ? mhdr->smac : ""),
	    hostname(x->flags, 4, sa),
	    portname(IS_LPORTS(uap->flags), "udp", udp->uh_sport));
	printf("> %s%s:%s udp",
	    ((uap->flags & FLINK) ? mhdr->dmac : ""),
	    hostname(x->flags, 4, da),
	    portname(IS_LPORTS(uap->flags), "udp", udp->uh_dport));
#endif
        if (frag) {
		printf( " %d:%d@%d%s\n",
		    ntohs(ip->ip_id), 
		    len, frag_off,
		    frag_off ? "" : "+");
        }
	else
		printf( " (DF)\n");
	if (!IS_FJHDR(x->flags))
		ascii_print_with_offset(d, len, 0x100, x->fd);
	uap->pcount += 1;
	return (0);
}
