/*
 *----------------------------------------------------------------------------
 * Copyright (c) 2001, 2002, C.S. Peron
 * All rights reserved.
 *----------------------------------------------------------------------------
 *
 *  * Redistribution and use in source and binary forms, with or without 
 *    modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice, this 
 *    list of conditions and the following disclaimer.
 *
 *  * Redistributions in binary form must reproduce the above 
 *    copyright notice, this list of conditions and the following disclaimer 
 *    in the documentation and/or other materials provided with the distribution.
 *
 *  * Neither the name of the ipex developers nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software 
 *    without specific prior written permission. 
 *
 *----------------------------------------------------------------------------
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *----------------------------------------------------------------------------
 */

#ifndef lint
static const char rcsid[] =
        "@(#) $Header: /usr/cvs/ipex24000/util.c,v 1.4 2004/08/05 23:17:48 modulus Exp $";
#endif

#include "ipex_includes.h"

void	reg_error(int msg, int fd);

/* XXX this is going to be nuked very soon */
/*
 * i initially wrote inet_write as a descriptor
 * based replacment of the printf(3) family of 
 * functions.
 */
int 
inet_write(register int fd, register const char *fmt, ...)
{
	char p[65535];
	register int err;
	va_list ap;
	static int len;

	va_start(ap, fmt);
	vsnprintf(p, 1024, fmt, ap);
	va_end(ap);
	len = strlen(p);
	if ((err = write(fd, p, len)) != len)
		return(-2);
	return (len);
}

/*
 * time_stamp() returns a pointer to 
 * a null terminated string. This 
 * string contains a timestamp of
 * arbitary precision. (nice precision too ;))
 */
char *
time_stamp(tvp, zone)
	register const struct timeval *tvp;
	int32_t zone;
{
	static char buf[32];
	struct tm *tm;
	register int s;
	char date[sizeof("XXXX-XX-XX")];

	tm = localtime((time_t *)&tvp->tv_sec);
	strftime(date, sizeof(date), "%Y-%m-%d", tm);
	s = (tvp->tv_sec + zone) % 86400;
	snprintf(buf, sizeof(buf),
	    "%s %02d:%02d:%02d.%06u", date,
	    (s / 3600), ((s % 3600) / 60),
	    (s % 60), ((u_int32_t)tvp->tv_usec));
	return (buf);
}

/*
 * this gmt2local() was taken from the
 * tcpdump(1) source code. thanks.
 */
int32_t 
gmt2local(t)
	time_t t;
{
	register int dt, dir;
	register struct tm *gmt, *loc;
	struct tm sgmt;

	if (t == 0) 
		t = time(0);
	gmt = &sgmt;
	*gmt = *gmtime(&t);
	loc = localtime(&t);
	dt = (loc->tm_hour - gmt->tm_hour) * 60 * 60 +
		(loc->tm_min - gmt->tm_min) *60;
	dir = loc->tm_year - gmt->tm_year;
	if (dir == 0)
		dir = loc->tm_yday - gmt->tm_yday;
	dt += dir * 24 * 60 * 60;
	return (dt);
}

/*
 * text_dump() is the function wich is responsible
 * for dumping the packet's contents in 
 * human readable form (where available).
 * it was borrowed from ngrep(1). again. thanks.
 */

void 
text_dump(data, len, fd)
	register char *data;
	register int len;
	int fd;
{
	if (uap->flags & FJHDR)
		return;
	printf(
	    "_______________________________________________________________________\n");
	if (len >  0) {
		unsigned width = 70;
		char *str = data;
		int j, i = 0;

		while (i < len) {
			putchar(' ');
			for(j = 0; j < width; j++)
				if (i + j < len) {
					printf("%c",
					    isprint(str[j])?str[j]:' ');
				} else 
					putchar(' ');
			str += width;
			i += j;
			putchar('\n');
		}
	}
	printf(
	    "-----------------------------------------------------------------------\n");
}

/*
 * this function returns a pointer to a 
 * null terminated string. for the use
 * of grep()'ing & dumping matched packets.
 */

char *
retbuf(p)
	register char *p;
{
	register int i;
	static char buf[2000];

	memset(buf, 0, sizeof(buf));
	for(i = 0; i < strlen(p); i++)
		snprintf(buf, sizeof(buf), "%s%c",
		    ((i == 0) ? "" : buf),
		    ((isprint(p[i]) ? p[i] : '.')));
	return (&buf[0]);
}

int
rettext(char *buffer, int len)
{
	int i;

	for(i = 0; i < len; i++)
		if (!isprint(buffer[i]))
			buffer[i] = ' ';
	return(0);
}

int 
grep(pattern, buf, fd)
	char *pattern;
	char *buf;
	int fd;
{
	regex_t rd;
	int flags, err, count;

	memset((char *)&rd, 0, sizeof(rd));
	flags = 0;
	flags |= REG_EXTENDED | REG_NOSUB | REG_ICASE;
	if ((err = regcomp(&rd, pattern, flags)) != 0) {
		reg_error(err, fd);
		exit(1);
	}
	if ((count = regexec(&rd, buf, (size_t)0, NULL, 0)) != 0) {
		regfree(&rd);
		return(1);
	}
	regfree(&rd);
	return(0);
}

void
reg_error(msg, fd)
        int msg;
	int fd;
{
        switch(msg) {
        case REG_NOMATCH:
                printf(
		"fatal: regexp: %d - regexec(3) failed to match (REG_NOMATCH)\n", msg);
                break;
        case REG_BADPAT:
                printf(
		"fatal: regexp: %d - invalid regular expression\n", msg);
                break;
        case REG_ECOLLATE:
                printf(
		"fatal: regexp: %d - invalid collating element\n", msg);
                break;
        case REG_ECTYPE:
                printf(
		"fatal: regexp: %d - invalid character class\n", msg);
                break;
	case REG_EESCAPE:
                printf(
		"fatal: regexp: %d - `\' applied to unescapable character\n", msg);
                break;
        case REG_ESUBREG:
                printf(
		"fatal: regexp: %d - invalid backreference number\n", msg);
                break;
        case REG_EBRACK:
                printf(
		"fatal: regexp: %d - brackets `[ ]' not balanced\n", msg);
                break;
        case REG_EPAREN:
                printf(
		"fatal: regexp: %d - parentheses `( )' not balanced\n", msg);
                break;
        case REG_EBRACE:
                printf(
		"fatal: regexp: %d - braces `{ }' not balanced\n", msg);
                break;
	case REG_BADBR:
                printf(
		"fatal: regexp: %d - invalid repetition count(s) in `{ }'\n", msg);
                break;
	case REG_ERANGE:
		printf(
		"fatal: regexp: %d - invalid character range in `[ ]'\n", msg);
                break;
	case REG_ESPACE:
                printf(
		"fatal: regexp: %d - ran out of memory\n", msg);
                break;
	case REG_BADRPT:
                printf(
		"fatal: regexp: %d - `?', `*', or `+' operand invalid\n", msg);
                break;
#ifndef linux
	case REG_EMPTY:
                printf(
		"fatal: regexp: %d - empty (sub)expression\n", msg);
                break;
	case REG_ASSERT:
		printf(
		"fatal: regexp: %d - can't happen - you found a bug\n", msg);
                break;
	case REG_INVARG:
                printf(
		"fatal: regexp: %d - invalid argument, e.g. negative-length string\n", msg);
                break;
#endif
	default:
                (void) fprintf(stderr,"fatal: regexp: *** unknown regexp error code ***\n");
                break;
        }

        return;
}

int
strtolower(string)
  register char *string;
{
  register char *c;

  for (c = string; *c; c++)
    if (!(*c = tolower (*c)))
      return (0);

  return (1);
}

int
strisdigit (string)
  register char *string;
{
   while(*string) {
      if (!isdigit(*string)) {
         return(1);
      }
      string++;
   }
   return(0);
}

#define IGNORE 0x1
#define SPACE ' '
#if (!defined(IS_IGNORING))
#define IS_IGNORING(a) ((a & IGNORE) ? (1) : (0))
#endif

/*
 * this function will return a null terminated
 * string to the contents located in the file
 * suppied by -f on the command line.
 * we will however parse out any comments that
 * may have been used in the file. Example
 * of legal comments:
 *
 * this is a body of text  ; this is ignored.
 * this is also a body of text # this is ignored.
 * standard C and C++ comments are also supported.
 */

void 
logic_load(char *p, FILE *fp)
{
   int flag = 0;
   char nwchar, nwchar_temp;
   char *op;

   op = p;
   while ((nwchar = getc(fp)) != EOF) {
      switch(nwchar) {
      case '#':
      case ';':
         flag |= IGNORE;
         break;
      case '/':
         nwchar_temp = getc(fp);
            if (nwchar_temp == '*') {
               while ((nwchar = getc(fp)) != EOF && nwchar_temp == '*') {
                  if (nwchar == '*') {
                     nwchar = getc(fp);
                        if (nwchar == '/')
                           nwchar_temp = '0';
                  }
               }
            } else if (nwchar_temp != '/') {
               *p = nwchar;
               p++;
               ungetc(nwchar_temp, fp);
            } else if (nwchar_temp == '/')
               flag |= IGNORE;
         break;
      case '\r':
         break;
      case '\n':
         if (IS_IGNORING(flag)) {
            flag &= ~IGNORE;
            break;
         }
         *p = SPACE;
         p++;
         break;
      default:
         if (IS_IGNORING(flag))
            break;
         *p = nwchar;
         p++;
         break;
      }
   }
   *p = 0;
   return;
}

void 
prescan_pcap_expr(char *expr)
{
   const char charset[]=
      "1234567890abcdefghijklmnopqrstuvwxyz"
      "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
   
   while(*expr) {
      if (strchr(charset, *expr)) {
         return;
      }
      expr++;
   }
   fprintf(stderr,"\n>> WARNING: NULL expr compiled: capturing ALL packets. <<\n\n");
   return;
}

char *
retrive_pcap_expr(char **argv)
{
   register char *p;
   static char pcap_expr[1000];

   if (*argv == NULL) {
      return(NULL);
   }
   while((p = *argv++)) {
      strncat(pcap_expr, p, sizeof(pcap_expr));
      strncat(pcap_expr, " ", 0x1);
   }
   return(pcap_expr);
}

/*
 * return a pointer to a null terminated string.
 * this string will be in xxx.xxx.xxx.xxx or
 * hostname form depending on the value of ``res''
 */

char    *hostname(res, v, ip)
int     res, v;
u_32_t  *ip;
{
# define MAX_INETA      16
   static char hname[MAXHOSTNAMELEN + MAX_INETA + 3];
   struct in_addr ipa;

   if (v == 4) {
      ipa.s_addr = *ip;
      if (IS_FPHEX(res)) {
         sprintf(hname, "0x%x", (unsigned)htonl(ipa.s_addr));
         return(hname);
      }
      if (IS_FPDNS(res)) {
         return(inet_ntoa(ipa));
      } else {
         return(cache_ns_entry(ip));
      }
   }
   return("IPv6");
}

/*
 * specify soft limit for cache
 */

#if (!defined(MAX_NS_CACHE))
#define MAX_NS_CACHE 524288	/* 1/2 Mb */
#endif

struct ns_entry {
   char alpha_host[MAXHOSTNAMELEN + MAX_INETA + 3];
   u_long ip_addr;
   struct ns_entry *next;
   struct ns_entry *prev;
};

struct ns_entry *NsHead;
struct ns_entry *NsTail;
unsigned long cached_nodes = 0;

char *cache_ns_entry(ip)
u_32_t *ip;
{
   struct ns_entry *new = NULL;
   struct hostent *hp;
   struct in_addr ipa;

   if (cached_nodes * sizeof(struct ns_entry) > MAX_NS_CACHE)
      return(inet_ntoa(ipa));

   ipa.s_addr = *ip;
   for(new = NsHead; new != NULL; new = new->next)
      if (ipa.s_addr == new->ip_addr)
         return(new->alpha_host);

   new = NULL; 
   new = memalloc(sizeof(struct ns_entry));
   if (new == NULL) {
      (void)fprintf(stderr,"fatal: exhausted memory.\n");
      exit(1);
   }

   hp = gethostbyaddr((char *)ip, sizeof(ip), AF_INET);
   if (!hp) {
      new->ip_addr = ipa.s_addr;
      strcpy(new->alpha_host, inet_ntoa(ipa));
      goto arrange;
   }

   new->ip_addr = ipa.s_addr;
   strcpy(new->alpha_host, hp->h_name);
   goto arrange;

arrange:
   if (NsHead == NULL) {
      NsHead = new;
      new->prev = NULL;
   } else {
      NsTail->next = new;
      new->prev    = NsTail;
   }
   NsTail = new;
   new->next = NULL;
   cached_nodes++;

   return(hp ? hp->h_name : new->alpha_host);
}

void flush_ns_cache(void)
{
   register struct ns_entry *node;
   register struct ns_entry *next_node;

   for(node = NsHead; node != NULL; node = next_node) {
      next_node = node->next;

      if (node->prev == NULL)
         NsHead = node->next;
      else
         node->prev->next = node->next;

      if (node->next == NULL)
         NsTail = node->prev;
      else
         node->next->prev = node->prev;

      free(node);
   }
   return;
}

void *
memalloc(unsigned int n)
{
	void *ret;

	if (!(ret = malloc(n)))
		errx(EX_UNAVAILABLE, "memory exhausted (failed to allocate" \
		    " %u bytes).", n);
	else
		return(ret);
}

void *
memrealloc(void *ptr, unsigned int n)
{
	void *ret;

	if (!(ret = realloc(ptr, n)))
		errx(EX_UNAVAILABLE, "memory exhausted (failed to reallocate" \
		    " %u bytes).", n);
	else
		return(ret);
}

/* EOF UTIL_C_ */
